/* 
 * RandMem Benchmark Version 1.2 Copyright (c) Roy Longbottom 2004 
 *
 * RandMem.exe carries out eight tests at increasing data sizes to produce 
 * data transfer rates in Mbytes Per Second from caches and memory. Serial and 
 * random address selections are employed, using the same program structure, 
 * with read and read/write tests for 32 bit integers and 64 bit floating 
 * point numbers. Results are displayed as the tests are running and appended 
 * to a log file, RandMem.txt. The benchmark runs via Win9X or NT/2K/XP.   
 *  
 * Memory demands double for each set of tests 6, 12, 24, 48 KB etc. up to
 * 40% of main RAM size. Random tests at the largest size can be slow, for
 * example 196 MB at 20 MB/sec. Minimum time is 100 milliseconds.
 *
 *  Modules
 *
 *  Config.cpp - via WhatConfig() identifies CPU type/speed, memory size
 *  and OS version,
 *
 *  RandMem.rc - menus and dialog boxes
 *     MENU_ABOUT    - AboutDlgProc()
 *     MENU_RUNALL   - runTests();
 *     MENU_DISPLAY  - displayWhatever()
 *     MENU_EXIT     - CleanUpAndPostQuit()
 *     DIALOG_8      - TestPanel()
 *     DIALOG_9      - RunPanel()
 *     AboutBox      - AboutDlgProc()
 *
 *  RandMem.h and RandMem.ico
 *
 *  RandMem.cpp
 *
 *  Functions
 *
 *  WinMain() - usual Windows functions
 *  InitApp() - usual Windows functions, allocates variable values,
 *     calculates maximum memory to be tested, uses WhatConfig()
 *     saveRand() and displayWhatever(), processes command line.
 *  WindowProc() - usual Windows functions for menus   
 *  AboutDlgProc() - displays About text
 *  CleanUpAndPostQuit() - set up exit
 *  TestPanel() - dialog box displayed at start with information
 *     including RAM size, two buttons:
 *         RUN_ALL as MENU_RUNALL
 *         EXIT_BUTTON as MENU_EXIT
 *     Displayed after tests showing results
 *  RunPanel() - dialog box cleared and refreshed after each test
 *      showing all results so far, no buttons
 *  displayWhatever() clears dialog boxes and displays TestPanel()
 *  runTests() - Opens log file, executes local_time(), logs date/time
 *  and headings, loop k for memory sizes, GlobalAlloc(), runs each
 *  of the following tests using checkTime(), logs speed results,
 *  destroys/creates RunPanel() for each test, GlobalFree(), logs
 *  end time 
 *    testII()   integer serial read
 *    testJI()   integer serial write/read    
 *    testIIR()  integer random read
 *    testJIR()  integer random write/read    
 *    testID()   floating point serial read
 *    testJD()   floating point serial write/read    
 *    testIDR()  floating point random read
 *    testJDR()  floating point random write/read
 *    all use start_time() and end_time()
 *  checkTime() - recalibrates jmax passes so tests run for at least
 *      runTime (100 milliseconds)
 *  local_time() - date/time of day
 *  start_time() - start time from high resolution timer
 *  end_time() - end time and running time msecs
 *  saveRand() - with command line option R saves random numbers
 *  used in RandData.txt
 *
 */
 
#define NAME "Test"
#define TITLE " Test Memory"
#define INITGUID

#include <windows.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <time.h>
#include <math.h>
#include "RandMem.h"



//  ************** PROTOPTYPES *****************

BOOL FAR PASCAL AboutDlgProc( HWND, UINT, WPARAM, LPARAM);
static BOOL InitApp(HINSTANCE, LPSTR, int);
long FAR PASCAL WindowProc(HWND, UINT, WPARAM, LPARAM);                  
static void CleanUpAndPostQuit(void);
void displayWhatever(HWND);

BOOL FAR PASCAL TestPanel( HWND, UINT, WPARAM, LPARAM);
BOOL FAR PASCAL RunPanel( HWND, UINT, WPARAM, LPARAM);

void local_time();
void end_time();
void start_time();
BOOL saveRand();

BOOL runTests();
int    testII(int jmax, int arraymax);
int    testIIR(int jmax, int arraymax);
int    testJI(int jmax, int arraymax);    
int    testJIR(int jmax, int arraymax);    
   
double testID(int jmax, int arraymax);    
double testIDR(int jmax, int arraymax);    
double testJD(int jmax, int arraymax);    
double testJDR(int jmax, int arraymax);    

int checkTime(int jmax);

// ************* GLOBAL VARIABLES **********
 
int    * xi;
double * xd;

char   version[30] = "Version 1.2";
double results[100][10];

BOOL bQuit = FALSE;

HWND   mainHWND;
HWND   testHwnd = NULL;
HWND   runHwnd = NULL;

HINSTANCE  MyInstance;

char    timeday[30];
LARGE_INTEGER starttime;
double   msecs;
double   runTime = 100;

char   writeMsg[256];

char    windowText[256];
char    commandLine[1024] = "";

short     keyPress;

int  ramKB;
int  ramKBused;
int  ramMB;
int  arraymax;
int  arraymax1;
int  darraymax;
int  passes = 1;
int  passesDone;
BOOL  runDone = FALSE;
BOOL  doTesting = FALSE;
BOOL  autoRun = FALSE;
int   randNumbers;
int   randMax;
char  startRam[200];

void displayWhatever(HWND hwnd)
{
    sprintf(windowText, " Random Memory Test");
    SetWindowText(mainHWND, windowText);        
        
    if (doTesting)
    {
        if (testHwnd)   DestroyWindow(testHwnd);
    }
    else
    {    
        if (runHwnd)    DestroyWindow(runHwnd);
        testHwnd = CreateDialog( MyInstance,"DIALOG_8", hwnd, (DLGPROC)TestPanel);
    }  
} // displayWhatever


// WinMain - initialization, message loop

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                     LPSTR cmdline, int cmdshow)
{
    MSG         msg;
    
    hPrevInstance = hPrevInstance;

    if (!InitApp(hInstance, cmdline, cmdshow))
         return FALSE;

    if (autoRun)
    {
         runTests();
         CleanUpAndPostQuit();
    }        
    while(!bQuit)
    {
        if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
        {
            
           if (msg.message == WM_QUIT)
           {
                CleanUpAndPostQuit();
                break;
           }
   
           if( !GetMessage( &msg, NULL, 0, 0 ) ) return msg.wParam;
           TranslateMessage(&msg);
           DispatchMessage(&msg);
        }
        else
        {
           // make sure we go to sleep if we have nothing else to do
           WaitMessage();
        }
    }
    return msg.wParam;
    
} // WinMain

// ***************************************************************************
//                    Initialization and object creation                     *
// ***************************************************************************

// InitApp
// Creates window

static BOOL
InitApp(HINSTANCE hInstance, LPSTR cmdline, int cmdshow)
{
    HWND hwnd;
    WNDCLASS wc;
    int   mult = 20;
    int   i, j, k;
    char *delims = {" ,"};
    char commands[20][256];
    char *p;
    int  KBytes;
    char buf[256] = "";    
  
    
    // set up and registers the window class
    
    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = WindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = sizeof(DWORD);
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(hInstance, "GenericIcon");
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName = "GenericMenu";
    wc.lpszClassName = NAME;
    if (!RegisterClass(&wc))
        return FALSE;

    // Create the window

    hwnd =
        CreateWindow(
            NAME,
            TITLE,
            WS_OVERLAPPEDWINDOW | WS_HSCROLL | WS_VSCROLL,
            0,
            0,
            640,
            480,
            NULL, 
            NULL, 
            hInstance, 
            NULL 
        );

    if (!hwnd) return FALSE;

    mainHWND = hwnd;
    strcpy(commandLine, strupr(cmdline));
    WhatConfig();
    strcpy(startRam, configData3);

    ramKB = ramsize / 1024;
    ramMB = ramKB / 1024;
    arraymax = 1536;
    arraymax1 = arraymax;

    i = 0;
    p = strtok(commandLine, delims);
    while (p != NULL)
    {
        strcpy(commands[i], p);
        strcat(buf, commands[i]);
        strcat(buf, " ");
        i = i + 1;
        if (i > 4) break;
        p = strtok(NULL, delims);
    }
    if (i > 0)
    {
        strcpy(commandLine, buf);
    }
    else
    {
        strcpy(commandLine, "None");
    }
    autoRun = FALSE;
    KBytes = ramMB * 1024;

    randNumbers = 0;
    randMax = 1000000;   
    for (j=0; j<i; j++)
    {
        switch(commands[j][0])
        {            
            case 'A': // Auto Run
             autoRun = TRUE;
             break;            

            case 'K': // KBytes
             if (j < i -1)
             {
                 sscanf(commands[j+1], "%d", &KBytes);
             }
             break;            

            case 'R': // Random numbers to file
             if (j < i -1)
             {
                 sscanf(commands[j+1], "%d", &randNumbers);
             }
             break;            

            case 'M': // Random number int multiplier
             if (j < i -1)
             {
                 sscanf(commands[j+1], "%d", &randMax);
             }
             break;            
        }
    }
   
    ramKBused = arraymax * 4 /1024;
    if (KBytes > ramMB * 768)
    {
        KBytes = 0;
    }
    else if (KBytes < ramKBused)
    {
        KBytes = 1;
    }        
    passes = 1;
    j = 0;    
    if (KBytes > 0)
    {
        do
        {
            ramKBused = ramKBused * 2;
            if (ramKBused > KBytes)
            {
                j = 1;
            }
            else
            {
                passes = passes + 1;
            }
        }
        while (j == 0);
        ramKBused = ramKBused / 2;          
    }
    else
    {
        if (ramMB > 96) mult = ramMB / 3;
    
        if (ramMB < 13)
        {
            passes = 9;
        }
        else if (ramMB < 17)
        {
            passes = 10;
        }
        else
        {
            passes = (int)(log((double)(ramKB - mult * 1024)/
                        ((double)(arraymax * 4) / 1024.0)) / log(2.0) + 1);
        }
        if (passes > 100) passes = 100; 
    
        ramKBused = arraymax;
        for (k=0; k<passes-1; k++)
        {
            ramKBused = ramKBused * 2;
        }
        ramKBused = ramKBused * 4 /1024;
    }
    keyPress = GetAsyncKeyState(VK_ESCAPE);

    // Display the window
    
    ShowWindow(hwnd, cmdshow);    
    UpdateWindow(hwnd);

    if(randNumbers > 0) saveRand();

    if (!autoRun) displayWhatever(mainHWND);
    
    return TRUE;
    
}  // InitApp



// WindowProc - handle messages for the main application window


long FAR PASCAL WindowProc(HWND hwnd, UINT msg,
                            WPARAM wparam, LPARAM lparam)                  
{
           
    switch( msg )
    {
    case WM_ACTIVATEAPP:
        break;
        
    case WM_INITMENUPOPUP:
        break;
        
    case WM_CREATE:
        break;

    case WM_DESTROY:
        CleanUpAndPostQuit();
        break;

    case WM_COMMAND:
        switch( LOWORD( wparam ) )
        {
        case MENU_ABOUT:
            DialogBox( MyInstance,"AboutBox", hwnd, (DLGPROC)AboutDlgProc );
            break;

       case MENU_RUNALL:
            doTesting = TRUE;
            displayWhatever(mainHWND);                                      
            runTests();
            break;


        case MENU_DISPLAY:
            displayWhatever(mainHWND);                                      
            break;
        
        case MENU_EXIT:
            CleanUpAndPostQuit();
            break;
                
        }
        break;
   
        case WM_PAINT:
             break;
           
        case WM_KEYDOWN:
            break;

       case WM_ACTIVATE:
            break;
    } 
    return DefWindowProc( hwnd, msg, wparam, lparam );

} // WindowProc




// AboutDlgProc - processes messages for the about dialog.

BOOL FAR PASCAL AboutDlgProc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam )
{
    lparam = lparam;

    switch( msg ) {
    case WM_INITDIALOG:
        return( TRUE );

    case WM_COMMAND:
        if( LOWORD( wparam ) == IDOK ) {
            EndDialog( hwnd, TRUE );
            return( TRUE );
        }
        break;
    }
    return( FALSE );

} // AboutDlgProc



  
void local_time()
{
    time_t time_of_day;
    time_of_day = time( NULL ); 
    strcpy (timeday, ctime(&time_of_day)); // timeday = date and time
    return;
}




// CleanUpAndPostQuit
// Release all D3DRM objects, post a quit message and set the bQuit flag

void
CleanUpAndPostQuit(void)
{
    bQuit = TRUE;
    return;
}


BOOL FAR PASCAL RunPanel( HWND hdlg, UINT msg,
                                WPARAM wparam, LPARAM lparam )
{
    wparam = wparam;
    lparam = lparam;
    int  k;
    char     tab = 9;
    int     arraymax2 = arraymax1;
    int FAR  tabs[2] = {1, 60};
    hdlg = hdlg;

     switch( msg )
     {
       case WM_INITDIALOG:
        sprintf(writeMsg, "Maximum RAM size to be tested %d KB", ramKBused);
        SendDlgItemMessage(hdlg, NOTES1, LB_ADDSTRING, 0, (LPARAM) writeMsg);
        sprintf(writeMsg, "Flashing disk light suggest insufficient RAM and paging activity "
                      "to/from RAM and disk.");
        SendDlgItemMessage(hdlg, NOTES1, LB_ADDSTRING, 0, (LPARAM) writeMsg);
        sprintf(writeMsg, "Press ESC to cancel and exit program. It might take some time.");
        SendDlgItemMessage(hdlg, NOTES1, LB_ADDSTRING, 0, (LPARAM) writeMsg);
        
        if (runDone)
        {
            sprintf(writeMsg, "%c%cMemory Random/Serial Speed Test %s", 9, 9, version);
            SendDlgItemMessage(hdlg, RESULTS1, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            SendDlgItemMessage(hdlg, RESULTS1, LB_ADDSTRING, 0, (LPARAM) "  ");
            sprintf(writeMsg, "   %cInteger.....................  %c %cDouble/Integer..............",
                                9 ,9, 9);
        
            SendDlgItemMessage(hdlg, RESULTS1, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            sprintf(writeMsg, "   %cSerial........ %cRandom........ %cSerial........ %cRandom........",
                                 9, 9, 9, 9);
            SendDlgItemMessage(hdlg, RESULTS1, LB_ADDSTRING, 0, (LPARAM) writeMsg);
        
            sprintf(writeMsg, "  RAM %cRead   %cRd/Wrt %cRead  %cRd/Wrt %cRead   %cRd/Wrt  %cRead   %cRd/Wrt",
                                 9, 9, 9, 9, 9, 9, 9, 9);
            SendDlgItemMessage(hdlg, RESULTS1, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            sprintf(writeMsg, "   KB %cMB/Sec %cMB/Sec %cMB/Sec %cMB/Sec %cMB/Sec %cMB/Sec %cMB/Sec %cMB/Sec",
                                 9, 9, 9, 9, 9, 9, 9, 9);
            SendDlgItemMessage(hdlg, RESULTS1, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            SendDlgItemMessage(hdlg, RESULTS1, LB_ADDSTRING, 0, (LPARAM) "  ");
            for (k=0; k<passesDone; k++)
            {
               sprintf(writeMsg, "%6d %c%7.0f %c%7.0f %c%7.0f %c%7.0f %c%7.0f %c%7.0f %c%7.0f %c%7.0f",  
                            arraymax2 * 4 / 1024, 9, results[k][0], 9, results[k][1], 9, results[k][2],
                                                 9, results[k][3], 9, results[k][4], 9, results[k][5],
                                                 9, results[k][6], 9, results[k][7] );
               SendDlgItemMessage(hdlg, RESULTS1, LB_ADDSTRING, 0, (LPARAM) writeMsg);
               SendDlgItemMessage(hdlg, RESULTS1, LB_SETCURSEL, k+7, 0L);
               arraymax2 = arraymax2 * 2;
            }
        }
        return( TRUE );
                    
     }
     return( FALSE );
    
} // runPanel

BOOL FAR PASCAL TestPanel( HWND hdlg, UINT msg,
                                WPARAM wparam, LPARAM lparam )
{
    wparam = wparam;
    lparam = lparam;
    int  k;
    char     tab = 9;
    int FAR  tabs[2] = {1, 60};
    hdlg = hdlg;

     switch( msg )
     {
       case WM_INITDIALOG:
        sprintf(writeMsg, "After running, results should be in file RandMem.txt and above");
        SendDlgItemMessage(hdlg, NOTES, LB_ADDSTRING, 0, (LPARAM) writeMsg);
        sprintf(writeMsg, "RAM size from GlobalMemoryStatus %d KB", ramKB);
        SendDlgItemMessage(hdlg, NOTES, LB_ADDSTRING, 0, (LPARAM) writeMsg);
        sprintf(writeMsg, "Maximum RAM size to be tested %d KB", ramKBused);
        SendDlgItemMessage(hdlg, NOTES, LB_ADDSTRING, 0, (LPARAM) writeMsg);

       
        if (runDone)
        {
            sprintf(writeMsg, "%c%cMemory Random/Serial Speed Test %s", 9, 9, version);
            SendDlgItemMessage(hdlg, RESULTS, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            SendDlgItemMessage(hdlg, RESULTS, LB_ADDSTRING, 0, (LPARAM) "  ");
            sprintf(writeMsg, "   %cInteger.....................  %c %cDouble/Integer..............",
                                9 ,9, 9);
        
            SendDlgItemMessage(hdlg, RESULTS, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            sprintf(writeMsg, "   %cSerial........ %cRandom........ %cSerial........ %cRandom........",
                                 9, 9, 9, 9);
            SendDlgItemMessage(hdlg, RESULTS, LB_ADDSTRING, 0, (LPARAM) writeMsg);
        
            sprintf(writeMsg, "  RAM %cRead   %cRd/Wrt %cRead  %cRd/Wrt %cRead   %cRd/Wrt  %cRead   %cRd/Wrt",
                                 9, 9, 9, 9, 9, 9, 9, 9);
            SendDlgItemMessage(hdlg, RESULTS, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            sprintf(writeMsg, "   KB %cMB/Sec %cMB/Sec %cMB/Sec %cMB/Sec %cMB/Sec %cMB/Sec %cMB/Sec %cMB/Sec",
                                 9, 9, 9, 9, 9, 9, 9, 9);
            SendDlgItemMessage(hdlg, RESULTS, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            SendDlgItemMessage(hdlg, RESULTS, LB_ADDSTRING, 0, (LPARAM) "  ");
            for (k=0; k<passesDone; k++)
            {
               sprintf(writeMsg, "%6d %c%7.0f %c%7.0f %c%7.0f %c%7.0f %c%7.0f %c%7.0f %c%7.0f %c%7.0f",  
                            arraymax * 4 / 1024, 9, results[k][0], 9, results[k][1], 9, results[k][2],
                                                 9, results[k][3], 9, results[k][4], 9, results[k][5],
                                                 9, results[k][6], 9, results[k][7] );
               SendDlgItemMessage(hdlg, RESULTS, LB_ADDSTRING, 0, (LPARAM) writeMsg);
               arraymax = arraymax * 2;
            }
        }
        else
        {
            strcpy(writeMsg, "Batch Command ");
            strncat(writeMsg, commandLine, 200);
            SendDlgItemMessage(hdlg, RESULTS, LB_ADDSTRING, 0, (LPARAM) writeMsg);
        }
        arraymax = arraymax1;

        return( TRUE );
                    
       case WM_COMMAND:

        if( HIWORD( wparam ) == BN_CLICKED)
        {
            switch (LOWORD (wparam))
            {
                case RUN_ALL:
                  PostMessage(mainHWND, WM_COMMAND, MENU_RUNALL, 0);
                 break;

                case EXIT_BUTTON:
                 PostMessage(mainHWND, WM_COMMAND, MENU_EXIT, 0);
                 break;
            }
        }

        return( TRUE );         
     }
     return( FALSE );
    
} // TestPanel


void start_time()
{
    QueryPerformanceCounter(&starttime);
}
 
void end_time()
{
    LARGE_INTEGER liDiff;
    LARGE_INTEGER liFreq;

    QueryPerformanceCounter(&liDiff);

    liDiff.QuadPart -= starttime.QuadPart;

    (void)QueryPerformanceFrequency(&liFreq);

    msecs = 1000.0 * (double)liDiff.QuadPart / (double) liFreq.QuadPart;
}


int testJIR(int jmax, int arraymax)    
{
    int i, j;
    int toti;
    
    start_time();
    
    for (j=0; j<jmax; j++)
    {
        toti = 0;
        for (i=0; i<arraymax; i=i+32)
        {
            xi[xi[i+28]] = xi[xi[i+30]];
            xi[xi[i+24]] = xi[xi[i+26]];
            xi[xi[i+20]] = xi[xi[i+22]];
            xi[xi[i+16]] = xi[xi[i+18]];
            xi[xi[i+12]] = xi[xi[i+14]];
            xi[xi[i+ 8]] = xi[xi[i+10]];
            xi[xi[i+ 4]] = xi[xi[i+ 6]];
            xi[xi[i+ 0]] = xi[xi[i+ 2]];
        }
    }
    end_time();
        
    return toti;
}

int testJI(int jmax, int arraymax)    
{
    int i, j;
    int toti;
    
    start_time();
    
    for (j=0; j<jmax; j++)
    {
        toti = 0;
        for (i=0; i<arraymax; i=i+32)
        {
            xi[xi[i+ 2]] = xi[xi[i+ 0]];
            xi[xi[i+ 6]] = xi[xi[i+ 4]];
            xi[xi[i+10]] = xi[xi[i+ 8]];
            xi[xi[i+14]] = xi[xi[i+12]];
            xi[xi[i+18]] = xi[xi[i+16]];
            xi[xi[i+22]] = xi[xi[i+20]];
            xi[xi[i+26]] = xi[xi[i+24]];
            xi[xi[i+30]] = xi[xi[i+28]];
        }
    }
    end_time();
        
    return toti;
}

int testIIR(int jmax, int arraymax)    
{
    int i, j;
    int toti;
    
    start_time();
    
    for (j=0; j<jmax; j++)
    {
        toti = 0;
        for (i=0; i<arraymax; i=i+32)
        {
            toti = toti & xi[xi[i+30]] | xi[xi[i+28]]
                        & xi[xi[i+26]] | xi[xi[i+24]]
                        & xi[xi[i+22]] | xi[xi[i+20]]
                        & xi[xi[i+18]] | xi[xi[i+16]]
                        & xi[xi[i+14]] | xi[xi[i+12]]
                        & xi[xi[i+10]] | xi[xi[i+ 8]]
                        & xi[xi[i+ 6]] | xi[xi[i+ 4]]
                        & xi[xi[i+ 2]] | xi[xi[i+ 0]];
        }
    }
    end_time();
        
    return toti;
}

int testII(int jmax, int arraymax)    
{
    int i, j;
    int toti;
    
    start_time();
    
    for (j=0; j<jmax; j++)
    {
        toti = 0;
        for (i=0; i<arraymax; i=i+32)
        {
            toti = toti & xi[xi[i+ 0]] | xi[xi[i+ 2]]
                        & xi[xi[i+ 4]] | xi[xi[i+ 6]]
                        & xi[xi[i+ 8]] | xi[xi[i+10]]
                        & xi[xi[i+12]] | xi[xi[i+14]]
                        & xi[xi[i+16]] | xi[xi[i+18]]
                        & xi[xi[i+20]] | xi[xi[i+22]]
                        & xi[xi[i+24]] | xi[xi[i+26]]
                        & xi[xi[i+28]] | xi[xi[i+30]];
        }
    }
    end_time();
        
    return toti;
}

double testJDR(int jmax, int darraymax)    
{
    int i, j;
    double totd;
        
    start_time();
    
    for (j=0; j<jmax; j++)
    {
        totd = 0.0;
        for (i=0; i<darraymax; i=i+16)
        {
            xd[xi[i+14]] = xd[xi[i+15]];
            xd[xi[i+12]] = xd[xi[i+13]];
            xd[xi[i+10]] = xd[xi[i+11]];
            xd[xi[i+ 8]] = xd[xi[i+ 9]];
            xd[xi[i+ 6]] = xd[xi[i+ 7]];
            xd[xi[i+ 4]] = xd[xi[i+ 5]];
            xd[xi[i+ 2]] = xd[xi[i+ 3]];
            xd[xi[i+ 0]] = xd[xi[i+ 1]];
        }
    }
    end_time();

    return totd;
}

double testJD(int jmax, int darraymax)    
{
    int i, j;
    double totd;
        
    start_time();
    
    for (j=0; j<jmax; j++)
    {
        totd = 0.0;
        for (i=0; i<darraymax; i=i+16)
        {
            xd[xi[i+ 1]] = xd[xi[i+ 0]];
            xd[xi[i+ 3]] = xd[xi[i+ 2]];
            xd[xi[i+ 5]] = xd[xi[i+ 4]];
            xd[xi[i+ 7]] = xd[xi[i+ 6]];
            xd[xi[i+ 9]] = xd[xi[i+ 8]];
            xd[xi[i+11]] = xd[xi[i+10]];
            xd[xi[i+13]] = xd[xi[i+12]];
            xd[xi[i+15]] = xd[xi[i+14]];
        }
    }
    end_time();

    return totd;
}

double testIDR(int jmax, int darraymax)    
{
    int i, j;
    double totd;
        
    start_time();
    
    for (j=0; j<jmax; j++)
    {
        totd = 0.0;
        for (i=0; i<darraymax; i=i+16)
        {
            totd = totd + xd[xi[i+15]] * xd[xi[i+14]]
                        - xd[xi[i+13]] * xd[xi[i+12]]
                        + xd[xi[i+11]] * xd[xi[i+10]]
                        - xd[xi[i+ 9]] * xd[xi[i+ 8]]
                        + xd[xi[i+ 7]] * xd[xi[i+ 6]]
                        - xd[xi[i+ 5]] * xd[xi[i+ 4]]
                        + xd[xi[i+ 3]] * xd[xi[i+ 2]]
                        - xd[xi[i+ 1]] * xd[xi[i+ 0]];
        }
    }
    end_time();

    return totd;
}

double testID(int jmax, int darraymax)    
{
    int i, j;
    double totd;
        
    start_time();
    
    for (j=0; j<jmax; j++)
    {
        totd = 0.0;
        for (i=0; i<darraymax; i=i+16)
        {
            totd = totd + xd[xi[i+ 0]] * xd[xi[i+ 1]]
                        - xd[xi[i+ 2]] * xd[xi[i+ 3]]
                        + xd[xi[i+ 4]] * xd[xi[i+ 5]]
                        - xd[xi[i+ 6]] * xd[xi[i+ 7]]
                        + xd[xi[i+ 8]] * xd[xi[i+ 9]]
                        - xd[xi[i+10]] * xd[xi[i+11]]
                        + xd[xi[i+12]] * xd[xi[i+13]]
                        - xd[xi[i+14]] * xd[xi[i+15]];
        }
    }
    end_time();

    return totd;
}


BOOL runTests()
{
    int i, k;
    int jmax;
    int toti;
    double totd;
    double sbytes;
    double mega = 1024.0 * 1024.0;
    char   addMsg[100];
    FILE    *outfile;

    outfile = fopen("RandMem.txt","a+");
    if (outfile == NULL)
    {
        sprintf(writeMsg, "Cannot open results file");
        if (!autoRun) MessageBox(NULL, writeMsg, "ERROR WILL EXIT", MB_ICONWARNING | MB_OK);
        CleanUpAndPostQuit();
        return FALSE;
    }

    local_time();
    
    fprintf(outfile, "\n   Memory Random/Serial Speed Test %s %s \n", version, timeday);
    fprintf(outfile, "                    Copyright (C) 2001, Roy Longbottom\n\n");

    fprintf(outfile, "         Integer.......................  Double/Integer................\n");
    fprintf(outfile, "         Serial........  Random........  Serial........  Random........\n");
    fprintf(outfile, "    RAM   Read   Rd/Wrt   Read   Rd/Wrt   Read   Rd/Wrt   Read   Rd/Wrt\n");
    fprintf(outfile, "     KB  MB/Sec  MB/Sec  MB/Sec  MB/Sec  MB/Sec  MB/Sec  MB/Sec  MB/Sec\n\n");
    fflush(outfile);
    
    runDone = TRUE;
    passesDone = 0;
    for (k=0; k<passes; k++) // 11
    {
        for (i=0; i<10; i++)
        {
            results[k][i] = 0;
        }
    }
    for (k=0; k<passes; k++) // 11
    {
        passesDone = k + 1;
        sprintf(writeMsg, "%6d ", arraymax * 4 / 1024);
        fprintf(outfile,  "%7d ", arraymax * 4 / 1024);

        xi = (int *)GlobalAlloc(GPTR, arraymax * 4 + 8);
        if (xi == NULL)
        {
            sprintf(writeMsg, "Cannot allocate memory");
            if (!autoRun) MessageBox(NULL, writeMsg, "ERROR WILL EXIT", MB_ICONWARNING | MB_OK);
            fprintf(outfile,"Cannot allocate memory\n\n");
            fflush(outfile);
            fclose(outfile);
            CleanUpAndPostQuit();
            return FALSE;
        }        
                        
        // Serial Read Integer *********************

        for (i=0; i<arraymax+1; i++)
        {
            xi[i] = i+1;
        }
        jmax = 1;
        sbytes = 4.0;
        do
        {
            toti = testII(jmax, arraymax);
            jmax = checkTime(jmax);   
        }
        while (msecs < runTime);
        results[k][0] =  (double)arraymax * (double)jmax * sbytes / mega / (msecs / 1000.0);

        if (runHwnd)    DestroyWindow(runHwnd);
        runHwnd = CreateDialog( MyInstance,"DIALOG_9", mainHWND, (DLGPROC)RunPanel);

        sprintf(addMsg, "%c%7.0f ", 9, results[k][0]);
        strcat(writeMsg, addMsg);
        fprintf(outfile, "%7.0f ", results[k][0]);
 
        if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
        {
            GlobalFree(xi);
            fprintf(outfile, "\n\n\n");
            fflush(outfile);
            fclose(outfile);
            CleanUpAndPostQuit();
            return FALSE;
        }

        // Serial Read/Write Integer *********************

        for (i=0; i<arraymax+1; i++)
        {
            xi[i] = i+1;
        }

        jmax = 1;
        sbytes = 4.0;
        do
        {
            toti = testJI(jmax, arraymax);    
            jmax = checkTime(jmax);   
        }
        while (msecs < runTime);
        results[k][1] =  (double)arraymax * (double)jmax * sbytes / mega / (msecs / 1000.0);
 
        if (runHwnd)    DestroyWindow(runHwnd);
        runHwnd = CreateDialog( MyInstance,"DIALOG_9", mainHWND, (DLGPROC)RunPanel);

        sprintf(addMsg, "%c%7.0f ", 9, results[k][1]);
        strcat(writeMsg, addMsg);
        fprintf(outfile, "%7.0f ", results[k][1]);

        if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
        {
            GlobalFree(xi);
            fprintf(outfile, "\n\n\n");
            fflush(outfile);
            fclose(outfile);
            CleanUpAndPostQuit();
            return FALSE;
        }

        // Random Read Integer *********************

        srand(123);

        for (i=0; i<arraymax+1; i++)
        {
            xi[i] = (int)((double)arraymax * ((double)(rand() * 32768 + rand()) / 32768 / 32768));
        }

        jmax = 1;
        sbytes = 4.0;
        do
        {
            toti = testIIR(jmax, arraymax);    
            jmax = checkTime(jmax);   
        }
        while (msecs < runTime);
        results[k][2] =  (double)arraymax * (double)jmax * sbytes / mega / (msecs / 1000.0);

        if (runHwnd)    DestroyWindow(runHwnd);
        runHwnd = CreateDialog( MyInstance,"DIALOG_9", mainHWND, (DLGPROC)RunPanel);

        sprintf(addMsg, "%c%7.0f ", 9, results[k][2]);
        strcat(writeMsg, addMsg);
        fprintf(outfile, "%7.0f ", results[k][2]);

        if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
        {
            GlobalFree(xi);
            fprintf(outfile, "\n\n\n");
            fflush(outfile);
            fclose(outfile);
            CleanUpAndPostQuit();
            return FALSE;
        }

        // Random Read/Write Integer *********************

        srand(123);
        for (i=0; i<arraymax+1; i++)
        {
            xi[i] = (int)((double)arraymax * ((double)(rand() * 32768 + rand()) / 32768 / 32768));
        }

        jmax = 1;
        sbytes = 4.0;
        do
        {
            toti = testJIR(jmax, arraymax);    
            jmax = checkTime(jmax);   
        }
        while (msecs < runTime);
        results[k][3] =  (double)arraymax * (double)jmax * sbytes / mega / (msecs / 1000.0);

        if (runHwnd)    DestroyWindow(runHwnd);
        runHwnd = CreateDialog( MyInstance,"DIALOG_9", mainHWND, (DLGPROC)RunPanel);

        sprintf(addMsg, "%c%7.0f ", 9, results[k][3]);
        strcat(writeMsg, addMsg);
        fprintf(outfile, "%7.0f ", results[k][3]);

        if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
        {
            GlobalFree(xi);
            fprintf(outfile, "\n\n\n");
            fflush(outfile);
            fclose(outfile);
            CleanUpAndPostQuit();
            return FALSE;
        }

        GlobalFree(xi);

        // xxxxxxxx Double

        darraymax = arraymax / 3;

        xi = (int    *)GlobalAlloc(GPTR, darraymax * 4 + 8);
        if (xi == NULL)
        {
            sprintf(writeMsg, "Cannot allocate memory");
            if (!autoRun) MessageBox(NULL, writeMsg, "ERROR WILL EXIT", MB_ICONWARNING | MB_OK);
            fprintf(outfile,"Cannot allocate memory\n\n");
            fflush(outfile);
            fclose(outfile);
            CleanUpAndPostQuit();
            return FALSE;
        }        
        xd = (double *)GlobalAlloc(GPTR, darraymax * 8 + 8);
        if (xd == NULL)
        {
            sprintf(writeMsg, "Cannot allocate memory");
            if (!autoRun) MessageBox(NULL, writeMsg, "ERROR WILL EXIT", MB_ICONWARNING | MB_OK);
            fprintf(outfile,"Cannot allocate memory\n\n");
            fflush(outfile);
            fclose(outfile);
            GlobalFree(xi);
            CleanUpAndPostQuit();
            return FALSE;
        }        


        // Serial Read Double *********************

        for (i=0; i<darraymax+1; i++)
        {
            xd[i] = 1.0;
            xi[i] = i+1;
        }
        jmax = 1;
        sbytes = 12.0;
        do
        {
            totd = testID(jmax, darraymax);       
            jmax = checkTime(jmax);   
        }
        while (msecs < runTime);
        results[k][4] =  (double)darraymax * (double)jmax * sbytes / mega / (msecs / 1000.0);

        if (runHwnd)    DestroyWindow(runHwnd);
        runHwnd = CreateDialog( MyInstance,"DIALOG_9", mainHWND, (DLGPROC)RunPanel);

        sprintf(addMsg, "%c%7.0f ", 9, results[k][4]);
        strcat(writeMsg, addMsg);
        fprintf(outfile, "%7.0f ", results[k][4]);

        if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
        {
            GlobalFree(xd);
            GlobalFree(xi);
            fprintf(outfile, "\n\n\n");
            fflush(outfile);
            fclose(outfile);
            CleanUpAndPostQuit();
            return FALSE;
        }

        // Serial Read/Write Double *********************

        for (i=0; i<darraymax+1; i++)
        {
            xd[i] = 1.0;
            xi[i] = i+1;
        }
        jmax = 1;
        sbytes = 12.0;
        do
        {
            totd = testJD(jmax, darraymax);    
            jmax = checkTime(jmax);   
        }
        while (msecs < runTime);
        results[k][5] =  (double)darraymax * (double)jmax * sbytes / mega / (msecs / 1000.0);

        if (runHwnd)    DestroyWindow(runHwnd);
        runHwnd = CreateDialog( MyInstance,"DIALOG_9", mainHWND, (DLGPROC)RunPanel);

        sprintf(addMsg, "%c%7.0f ", 9, results[k][5]);
        strcat(writeMsg, addMsg);
        fprintf(outfile, "%7.0f ", results[k][5]);

        if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
        {
            GlobalFree(xd);
            GlobalFree(xi);
            fprintf(outfile, "\n\n\n");
            fflush(outfile);
            fclose(outfile);
            CleanUpAndPostQuit();
            return FALSE;
        }

        // Random Read Double *********************
        
        srand(123);
        for (i=0; i<darraymax+1; i++)
        {
            xi[i] = (int)((double)darraymax * ((double)(rand() * 32768 + rand()) / 32768 / 32768));
            xd[i] = 1.0;
        }

        jmax = 1;
        sbytes = 12.0;    
        do
        {
            totd = testIDR(jmax, darraymax); 
            jmax = checkTime(jmax);   
        }
        while (msecs < runTime);
        results[k][6] =  (double)darraymax * (double)jmax * sbytes / mega / (msecs / 1000.0);

        if (runHwnd)    DestroyWindow(runHwnd);
        runHwnd = CreateDialog( MyInstance,"DIALOG_9", mainHWND, (DLGPROC)RunPanel);

        sprintf(addMsg, "%c%7.0f ", 9, results[k][6]);
        strcat(writeMsg, addMsg);
        fprintf(outfile, "%7.0f ", results[k][6]);

        if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
        {
            GlobalFree(xd);
            GlobalFree(xi);
            fprintf(outfile, "\n\n\n");
            fflush(outfile);
            fclose(outfile);
            CleanUpAndPostQuit();
            return FALSE;
        }

        // Random Read/Write Double *********************

        srand(123);
        for (i=0; i<darraymax+1; i++)
        {
            xi[i] = (int)((double)darraymax * ((double)(rand() * 32768 + rand()) / 32768 / 32768));
        }

        jmax = 1;
        sbytes = 12.0;    
        do
        {
            totd = testJDR(jmax, darraymax); 
            jmax = checkTime(jmax);   
        }
        while (msecs < runTime);
        results[k][7] =  (double)darraymax * (double)jmax * sbytes / mega / (msecs / 1000.0);

        if (runHwnd)    DestroyWindow(runHwnd);
        runHwnd = CreateDialog( MyInstance,"DIALOG_9", mainHWND, (DLGPROC)RunPanel);

        sprintf(addMsg, "%c%7.0f ", 9, results[k][7]);
        strcat(writeMsg, addMsg);
        fprintf(outfile, "%7.0f", results[k][7]);

        if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
        {
            GlobalFree(xd);
            GlobalFree(xi);
            fprintf(outfile, "\n\n\n");
            fflush(outfile);
            fclose(outfile);
            CleanUpAndPostQuit();
            return FALSE;
        }
        WhatConfig();

        GlobalFree(xd);
        GlobalFree(xi);

        fprintf(outfile, "\n");
        arraymax = arraymax * 2;
        fflush(outfile);

        if (runHwnd)    DestroyWindow(runHwnd);
        runHwnd = CreateDialog( MyInstance,"DIALOG_9", mainHWND, (DLGPROC)RunPanel);
    }
    if (autoRun) DestroyWindow(runHwnd);
    arraymax = arraymax1;
    local_time();

    fprintf(outfile, "\n                End of test %s\n", timeday);        
    fprintf(outfile," %s\n", configData1);
    fprintf(outfile," %s\n", configData2);
    fprintf(outfile," %s, at end %d KB\n", startRam, freesize / 1024);        
    fprintf(outfile,"\n\n");
    fclose(outfile);
    doTesting = FALSE;
    if (!autoRun) displayWhatever(mainHWND);                                      

    return TRUE;
    
} // runTests

BOOL saveRand()
{
    int    i, j;
    int    count[12];
    int    gotit;
    int    *r;
    FILE    *outfile2;
    HDC     hdc;

    hdc = GetDC(mainHWND);
    sprintf(writeMsg, "                          Allocating Memory  ");
    TextOut(hdc, 100, 100, writeMsg, lstrlen(writeMsg));    

    r = (int *)GlobalAlloc(GPTR, randNumbers * 4);
        
    outfile2 = fopen("RandData.txt","w");
    srand(123);

    for (i=0; i<randNumbers; i++)
    {
        r[i] = (int)((double)randMax * ((double)(rand() * 32768 + rand()) / 32768 / 32768));
    }

    if(randNumbers >= randMax)
    {
        fprintf(outfile2, " %d random numbers %d max possible\n\n", randNumbers, randMax);
    
        for (i=0; i<12; i++)
        {
            count[i] = 0;
        }
        sprintf(writeMsg, "                 Random Number Check         ");
        TextOut(hdc, 100,  60, writeMsg, lstrlen(writeMsg));    
        sprintf(writeMsg, "             Press Esc to Stop Calculations     ");
        TextOut(hdc, 100, 140, writeMsg, lstrlen(writeMsg));    
        sprintf(writeMsg, "                Results in RandData.txt         ");
        TextOut(hdc, 100, 180, writeMsg, lstrlen(writeMsg));    
        for (i=0; i<randMax; i++)
        {
            sprintf(writeMsg, "         Compare Loop %d out of %d          ", i+1, randMax);
            TextOut(hdc, 100, 100, writeMsg, lstrlen(writeMsg));    
            if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
            {
                fclose(outfile2);
                GlobalFree(r);
                ReleaseDC(mainHWND, hdc);
                return FALSE;    
            }        
            gotit = 0;
            for (j=0; j<randNumbers; j++)
            {
                if(r[j] == i) gotit = gotit + 1;
            }
            if(gotit < 11)
            {
                count[gotit] = count[gotit] + 1;
            }
            else
            {
                count[11] = count[11] + 1;
            }
        }
        fprintf(outfile2, " Appearences  Count \n\n");
        for (i=0; i<11; i++)
        {
            fprintf(outfile2, " %7d %10d\n", i, count[i]); 
        }
        fprintf(outfile2, "     >10 %10d\n\n", count[11]);
    } 
    sprintf(writeMsg, "                 Writing Output File         ");
    TextOut(hdc, 100, 2200, writeMsg, lstrlen(writeMsg));    

    for (i=0; i<randNumbers; i++)
    {
        fprintf(outfile2, "%d\n", r[i]);
    }
        
    fclose(outfile2);
    GlobalFree(r);
    ReleaseDC(mainHWND, hdc);
    return TRUE;
        
} // saveRand

int checkTime(int jmax)
{
    if (msecs < runTime)
    {
          if (msecs < runTime / 8.0)
          {
                jmax = jmax * 10;
          }
          else
          {
                jmax = (int)(runTime * 1.25 / msecs * (double)jmax+1);
          }
      }
      return jmax;
}


//    MessageBox(NULL, writeMsg, "WARNING", MB_ICONWARNING | MB_OK);

//         xi[i] = (int)((double)arraymax * ((double)rand() / 32767) );

//    (int)((double)randMax * ((double)rand() / 32767) );
